#ifndef IO_IMAGE__HH
#define IO_IMAGE__HH

#ifdef WIN32
#include <iostream>
#else
#include <iostream.h>
#endif
using namespace std;

#include "image.H"
#include "imagetools.H" //Dfinition du type RGB

/** @name Classe entree/sortie Image
    @memo Classe regroupant toutes les fonctions necessaires a la lecture et ecriture d'images.
    @doc Classe offrant des possibilites de lecture et ecriture pour les differents formats de fichiers.
    Les fonctions etant  statiques, il est inutile de declarer un objet de type ImageIO.
    Toute fonction renvoie un entier non nul en cas de reussite.\\
    {\bf Formats supportes:}
    \begin{itemize}
    \item raw sans entete (unsigned char, short et autres)
    \item avec entete pgm (unsigned char, short et autres)
    \item ccett (uniquement en lecture)
    \end{itemize}
    \\
    {\bf Fichier a inclure :}
    \begin{verbatim}
    #include "imageio.H"
    \end{verbatim}
    \\
    {\bf Exemple :}
    \begin{verbatim}
    #include "image.H"
    #include "imageio.H"
    #include "imagedef.H"

    int main (void)
    {
      // lecture ecrite PGM
      Image<octet> im1;
      ImageIO<octet>::ReadPGM("toto.pgm",im1);
      ImageIO<octet>::WritePGM("titi.pgm",im1);

      // lecture ecrite PPM
      Image<RGB> im2;
      ImageIO<RGB>::ReadPPM("toto.ppm",im2);
      ImageIO<RGB>::WritePPM("titi.ppm",im2);

      // lecture ecrite RAW
      Image<octet> im3;
      ImageIO<octet>::ReadRawType("toto.raw",im3,384,288);
      ImageIO<octet>::WritePGM("toto2.pgm",im3);
    }
    \end{verbatim}
    @author \URL[Thomas Guionnet]{mailto:Thomas.Guionnet@irisa.fr}
    @author \URL[Stephane Pateux]{mailto:Stephane.Pateux@irisa.fr}
    @author \URL[Franck Galpin]{mailto:Franck.Galpin@irisa.fr}
    @see Image ImageTools
    @version base sur IO_Image par \URL[Stephane Pateux]{mailto:Stephane.Pateux@irisa.fr}
*/
//@{

/** Classe ImageIO */
template <class T>
class ImageIO {
public:

  /** @name Fontions usuelles (lecture/ecriture PGM/PPM/RAW) */
  //@{
  /** @doc lecture d'une image pgm
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGM(const char *const filename, Image<T> &image);

  /** @doc lecture d'une image ppm
      @param filename nom du fichier a lire
      @param imageR image de stockage de la composante R
      @param imageG image de stockage de la composante G
      @param imageB image de stockage de la composante B
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPPM(const char *const filename, Image<T> &imageR,Image<T> &imageG,Image<T> &imageB);

  /** @doc lecture d'une image ppm (n'est instancie que pour le type image<RGB>)
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  //static int ReadPPM(const char *const filename, Image<T> &image) { exit(-1); return(0); };
  static int ReadPPM(const char *const filename,  Image<RGB> &image);


  /** @doc lecture d'une image sans entete avec donnees templatees
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawType(const char *const filename, Image<T> &image, const int largeur, const int hauteur);

  /** @doc ecriture d'une image pgm
      @param filename nom du fichier a ecrire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGM(const char *const filename, const Image<T> &image);

  /** @doc ecriture d'une image ppm
      @param filename nom du fichier a ecrire
      @param imageR image de la composante R
      @param imageG image de la composante G
      @param imageB image de la composante B
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePPM(const char *const filename, const Image<T> &imageR,const Image<T> &imageG,const Image<T> &imageB);

  /** @doc ecriture d'une image ppm (n'est instancie que pour le type image<RGB>)
      @param filename nom du fichier a ecrire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  //static int WritePPM(const char *const filename, const Image<T> &image) { exit(-1); return(0); };
  static int WritePPM(const char *const filename, const Image<RGB> &image);

  /** @doc ecriture d'une image en raw (sans entete)
      @param filename nom du fichier a ecrire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawType(const char *const filename, const Image<T> &image);
  //@}

  /**@name Fonctions de lecture specialisees */
  //@{
  /** @doc lecture d'une image de style pgm mais dont les donnees sont de type T
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGMType(const char *const filename, Image<T> &image);

  /** @doc lecture d'une image sans entete  dont les donnees sont de type unsigned char
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawUChar(const char *const filename, Image<T> &image, const int largeur, const int hauteur);

  /** @doc lecture d'une image sans entete dont les donnees sont de type short
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawShort(const char *const filename, Image<T> &image, const int largeur, const int hauteur);

  /** @doc lecture d'une image pgm (equivalent a ReadPGM)
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGMUChar(const char *const filename, Image<T> &image);

  /** @doc lecture d'une image sans entete dont les donnees sont de type RGB stockees en unsigned char
      @param filename nom du fichier a lire
      @param imageR image de stockage de la composante R
      @param imageG image de stockage de la composante G
      @param imageB image de stockage de la composante B
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawRGBUChar(const char *const filename, Image<T> &imageR,Image<T> &imageG,Image<T> &imageB, int largeur, int hauteur);

  /** @doc lecture d'une image de style pgm mais dont les donnees sont de type short
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGMShort(const char *const filename, Image<T> &image);

  /** @doc lecture d'une image dans une sequence CCETT
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @param numImage numero de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadImageCCETT(const char *const filename, Image<T> &image, int numImage=1);

  /** @doc lecture d'une trame dans une sequence CCETT
      @param filename nom du fichier a lire
      @param image image de stockage de l'image lue
      @param numTrame numero de la trame a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadTrameCCETT(const char *const filename, Image<T> &image, int numTrame=1);
  //@}

  /**@name Fonctions d'ecriture specialisees */
  //@{
  /** @doc ecriture d'une image sans entete  dont les donnees sont de type unsigned char
      @param filename nom du fichier a lire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawUChar(const char *const filename, const Image<T> &image);

  /** @doc ecriture d'une image sans entete  dont les donnees sont de type short
      @param filename nom du fichier a lire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawShort(const char *const filename, const Image<T> &image);

  /** @doc ecriture d'une image dans le style pgm  dont les donnees sont stockees en type T
      @param filename nom du fichier a lire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGMType(const char *const filename, const Image<T> &image);

  /** @doc ecriture d'une image dans le style pgm  dont les donnees sont stockees en unsigned char
      @param filename nom du fichier a lire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGMUChar(const char *const filename, const Image<T> &image);

  /** @doc ecriture d'une image sans entete dont les donnees sont stockees en 3 composantes unsigned char entrelacees
      @param filename nom du fichier a lire
      @param imageR image de la composante R a ecrire
      @param imageG image de la composante G a ecrire
      @param imageB image de la composante B a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawRGBUChar(const char *const filename, const Image<T> &imageR,const Image<T> &imageG,const Image<T> &imageB);

  /** @doc ecriture d'une image dans le style pgm  dont les donnees sont stockees en short
      @param filename nom du fichier a lire
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGMShort(const char *const filename, const Image<T> &image);
  //@}

  /**@name Fonctions de lecture dans des flux */
  //@{
  /** @doc lecture d'une image sans entete avec donnees templatees
      @param is flux d'entree
      @param image image de stockage de l'image lue
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawType(istream &is, Image<T> &image, const int largeur, const int hauteur);

  /** @doc lecture d'une image sans entete  dont les donnees sont de type unsigned char
      @param is flux d'entree
      @param image image de stockage de l'image lue
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawUChar(istream &is, Image<T> &image, const int largeur, const int hauteur);

  /** @doc lecture d'une image sans entete dont les donnees sont de type short
      @param is flux d'entree
      @param image image de stockage de l'image lue
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawShort(istream &is, Image<T> &image, const int largeur, const int hauteur);

  /** @doc lecture d'une image de style pgm mais dont les donnees sont de type T
      @param is flux d'entree
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGMType(istream &is, Image<T> &image);

  /** @doc lecture d'une image pgm (equivalent a ReadPGM)
      @param is flux d'entree
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGMUChar(istream &is, Image<T> &image);

  /** @doc lecture d'une image pgm
      @param is flux d'entree
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGM(istream &is,  Image<T> &image) {  return ReadPGMUChar(is, image); }

  /** @doc lecture d'une image sans entete dont les donnees sont de type RGB stockees en unsigned char
      @param is flux d'entree
      @param imageR image de stockage de la composante R
      @param imageG image de stockage de la composante G
      @param imageB image de stockage de la composante B
      @param largeur largeur de l'image a lire
      @param hauteur hauteur de l'image a lire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadRawRGBUChar(istream &is, Image<T> &imageR,Image<T> &imageG,Image<T> &imageB, int largeur, int hauteur);

  /** @doc lecture d'une image ppm
      @param is flux d'entree
      @param imageR image de stockage de la composante R
      @param imageG image de stockage de la composante G
      @param imageB image de stockage de la composante B
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPPM(istream &is, Image<T> &imageR,Image<T> &imageG,Image<T> &imageB);

  /** @doc lecture d'une image de style pgm mais dont les donnees sont de type short
      @param is flux d'entree
      @param image image de stockage de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int ReadPGMShort(istream &is, Image<T> &image);
  //@}

  /**@name Fonctions d'ecriture dans des flux */
  //@{
  /** @doc ecriture d'une image en raw (sans entete)
      @param os flux de sortie
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawType(ostream &os, const Image<T> &image);

  /** @doc ecriture d'une image sans entete  dont les donnees sont de type unsigned char
      @param os flux de sortie
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawUChar(ostream &os, const Image<T> &image);


   /** @doc ecriture d'une image sans entete  dont les donnees sont de type short
       @param os flux de sortie
       @param image image a ecrire
       @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawShort(ostream &os, const Image<T> &image);

  /** @doc ecriture d'une image dans le style pgm  dont les donnees sont stockees en type T
      @param os flux de sortie
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGMType(ostream &os, const Image<T> &image);

  /** @doc ecriture d'une image dans le style pgm  dont les donnees sont stockees en unsigned char
      @param os flux de sortie
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGMUChar(ostream &os, const Image<T> &image);

  /** @doc ecriture d'une image pgm
      @param os flux de sortie
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGM(ostream &os, const Image<T> &image) {  return WritePGMUChar(os, image); }

  /** @doc ecriture d'une image sans entete dont les donnees sont stockees en 3 composantes unsigned char entrelacees
      @param os flux de sortie
      @param imageR image de la composante R a ecrire
      @param imageG image de la composante G a ecrire
      @param imageB image de la composante B a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WriteRawRGBUChar(ostream &os, const Image<T> &imageR,const Image<T> &imageG,const Image<T> &imageB);

  /** @doc ecriture d'une image ppm
      @param os flux de sortie
      @param imageR image de la composante R
      @param imageG image de la composante G
      @param imageB image de la composante B
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePPM(ostream &os, const Image<T> &imageR,const Image<T> &imageG,const Image<T> &imageB);

  /** @doc ecriture d'une image dans le style pgm  dont les donnees sont stockees en short
      @param os flux de sortie
      @param image image a ecrire
      @return 0 en cas d'echec, 1 en cas de succes */
  static int WritePGMShort(ostream &os, const Image<T> &image);
  //@}

  /**@name Fonctions de lecture des infos d'une image PGM/PPM/CCETT */
  //@{
  /** @doc permet d'obtenir la taille d'une image PGM
      @param is flux d'entree
      @param largeur largeur de l'image lue
      @param hauteur hauteur de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int InfoImagePGM(istream &is, int &largeur, int &hauteur);

  /** @doc permet d'obtenir la taille d'une image PGM
      @param filename nom du fichier a lire
      @param largeur largeur de l'image lue
      @param hauteur hauteur de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int InfoImagePGM(const char *const filename, int &largeur, int &hauteur);

  /** @doc permet d'obtenir la taille d'une image PPM
      @param is flux d'entree
      @param largeur largeur de l'image lue
      @param hauteur hauteur de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int InfoImagePPM(istream &is, int &largeur, int &hauteur);

  /** @doc permet d'obtenir la taille d'une image PPM
      @param filename nom du fichier a lire
      @param largeur largeur de l'image lue
      @param hauteur hauteur de l'image lue
      @return 0 en cas d'echec, 1 en cas de succes */
  static int InfoImagePPM(const char *const filename, int &largeur, int &hauteur);

  /** @doc permet d'obtenir la taille d'une image dans une sequence CCETT
      @param filename nom du fichier a lire
      @param largeur largeur de l'image lue
      @param hauteur hauteur de l'image lue
      @param nbImage nombre d'image dans la sequence
      @return 0 en cas d'echec, 1 en cas de succes */
  static int InfoSequenceCCETT(const char *const filename, int &largeur, int &hauteur, int &nbImage);
  //@}
protected:
  /**@name Fonctions privees de manipulation de flux */
  //@{
  /** @doc permet de sauter les espaces superflus dans une image PGM
      @param is flux d'entree */
  static void SkipPGMWhiteSpace(istream &is);

  /** @doc permet de lire l'entete d'une image PGM
      @param is flux d'entree
      @param largeur largeur de l'image lue
      @param hauteur hauteur de l'image lue */
  static int ReadEntetePGM(istream &is, int &largeur, int &hauteur);

  /** @doc permet de lire l'entete d'une image PPM
      @param is flux d'entree
      @param largeur largeur de l'image lue
      @param hauteur hauteur de l'image lue */
  static int ReadEntetePPM(istream &is, int &largeur, int &hauteur);

  /** @doc permet d'ecrire l'entete d'une image PGM
      @param os flux de sortie
      @param largeur largeur de l'image
      @param hauteur hauteur de l'image */
  static int WriteEntetePGM(ostream &os, const int largeur, const int hauteur);

  /** @doc permet d'ecrire l'entete d'une image avec le style PGM
      @param os flux de sortie
      @param largeur largeur de l'image
      @param hauteur hauteur de l'image */
  static int WriteEntetePseudoPGM(ostream &os, const int largeur, const int hauteur);

  /** @doc permet d'ecrire l'entete d'une image PPM
      @param os flux de sortie
      @param largeur largeur de l'image
      @param hauteur hauteur de l'image */
  static int WriteEntetePPM(ostream &os, const int largeur, const int hauteur);
  //@}
};
//@}
#include "imageio.cpp"

#endif // IO_IMAGE__HH


